package jp.co.yahoo.hadoop.ml.aozora;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import net.reduls.gomoku.Morpheme;
import net.reduls.gomoku.Tagger;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.BytesWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * A mapper for extracting the features of books of Aozora-Bunko.
 */
public class ExtractAozoraFeatureMapper extends
		Mapper<NullWritable, BytesWritable, VIntWritable, MapWritable> {
	Map<String, Integer> lifeMap = new HashMap<String, Integer>();

	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		String lifeFile = conf.get("author.classifier.life.file");
		FileSystem fs = FileSystem.get(conf);
		FSDataInputStream is = fs.open(new Path(lifeFile));
		BufferedReader br = new BufferedReader(new InputStreamReader(is,
				"UTF-8"));
		try {
			String line = null;
			while ((line = br.readLine()) != null) {
				String[] items = line.split(",");
				lifeMap.put(items[0], Integer.parseInt(items[1]));
			}
		} finally {
			br.close();
			is.close();
		}
	}

	@Override
	public void map(NullWritable key, BytesWritable value, Context context)
			throws IOException, InterruptedException {
		String rawText = new String(value.getBytes(), "Shift_JIS");
		Aozora aozora = AozoraParser.parse(rawText);
		Integer life = lifeMap.get(aozora.getAuthor());
		if (life != null) {
			MapWritable wordFreqMap = new MapWritable();
			for (Morpheme m : Tagger.parse(aozora.getNormalizedBody())) {
				Text word = new Text(m.surface);
				if (!CommonUtil.isContentWord(m.feature))
					continue;
				VIntWritable freq = (VIntWritable) wordFreqMap.get(word);
				if (freq == null) {
					wordFreqMap.put(word, new VIntWritable(1));
				} else {
					freq.set(freq.get() + 1);
				}
			}
			context.write(new VIntWritable(life), wordFreqMap);
		}
	}
}
