package jp.co.yahoo.hadoop.ml.classifier;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * A mapper for multi-class classification
 */
public abstract class MulticlassClassifierMapper extends
		Mapper<VIntWritable, MapWritable, VIntWritable, MapWritable> {
	protected Map<VIntWritable,MapWritable> weightMap = new HashMap<VIntWritable,MapWritable>();
	protected int classNum = 20;
	protected int classWidth = 5;

	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		
		classNum = conf.getInt("class.num", classNum);
		classWidth = conf.getInt("class.width", classWidth);
		
		String weightFile = conf.get("weight.file");
		if (weightFile == null){
			for(int i=0; i < classNum; i++){
				weightMap.put(new VIntWritable(i), new MapWritable());
			}
		}else{
			FileSystem fs = FileSystem.get(conf);
			SequenceFile.Reader reader = new SequenceFile.Reader(fs, new Path(
					weightFile), conf);
			try {
				VIntWritable key = new VIntWritable();
				MapWritable value = new MapWritable();
				while(reader.next(key, value)){
					weightMap.put(new VIntWritable(key.get()), new MapWritable(value));
				}
			} finally {
				reader.close();
			}
		}
	}
	
	@Override
	public void cleanup(Context context) throws IOException,
			InterruptedException {
		for ( VIntWritable key : weightMap.keySet()){
			MapWritable value = weightMap.get(key);
			context.write(key, value);
		}
	}
}