package jp.co.yahoo.hadoop.ml.classifier;

import java.io.IOException;

import jp.co.yahoo.hadoop.ml.common.IterativeParameterMixingDriver;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.SequenceFileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.SequenceFileOutputFormat;

/**
 * A MapReduce driver for multi-class classification
 */
public class MulticlassIterativeParameterMixingDriver extends
		IterativeParameterMixingDriver {

	public static void main(String[] args) throws Exception {
		MulticlassIterativeParameterMixingDriver driver = new MulticlassIterativeParameterMixingDriver();
		driver.run(args);
	}

	@Override
	protected void runIteration(Configuration conf, Path input, Path output,
			String algorithm) throws IOException, InterruptedException,
			ClassNotFoundException {
		Job job = new Job(conf, algorithm + ": " + output);
		job.setJarByClass(IterativeParameterMixingDriver.class);
		job.setMapperClass(getMapperClass(algorithm));
		job.setReducerClass(MulticlassClassifierReducer.class);
		job.setInputFormatClass(SequenceFileInputFormat.class);
		job.setOutputFormatClass(SequenceFileOutputFormat.class);
		job.setOutputKeyClass(VIntWritable.class);
		job.setOutputValueClass(MapWritable.class);
		job.setNumReduceTasks(1);
		FileInputFormat.addInputPath(job, input);
		FileOutputFormat.setOutputPath(job, output);

		if (!job.waitForCompletion(true)) {
			throw new InterruptedException("job failed: " + output);
		}
	}
	
	private Class<? extends Mapper> getMapperClass(String algorithm) {
		Class<? extends Mapper> resultClass = MulticlassLogisticRegressionMapper.class;
		return resultClass;
	}
}
