package jp.co.yahoo.hadoop.ml.classifier;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import jp.co.yahoo.hadoop.ml.common.DiscriminantFunctionAlgorithm;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * A mapper for the test of multi-class classification
 */
public class TestMulticlassClassifierMapper extends Mapper<VIntWritable, MapWritable, IntWritable, DoubleWritable>{
	private Map<VIntWritable,MapWritable> weightMap = new HashMap<VIntWritable,MapWritable>();
	private int trainSize = 0;
	private double diffSum = 0;
	private int classWidth = 5;
	
	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		classWidth = conf.getInt("class.width", classWidth);
		String weightFile = conf.get("weight.file");
		if (weightFile != null) {
			FileSystem fs = FileSystem.get(conf);
			SequenceFile.Reader reader = new SequenceFile.Reader(fs, new Path(
					weightFile), conf);
			try {
				VIntWritable key = new VIntWritable();
				MapWritable value = new MapWritable();
				while(reader.next(key, value)){
					weightMap.put(new VIntWritable(key.get()), new MapWritable(value));
				}
			} finally {
				reader.close();
			}
		}
	}
	
	@Override
	public void map(VIntWritable key, MapWritable value, Context context)
			throws IOException, InterruptedException {
		int b = key.get();
		value.put(new Text(CommonUtil.BIAS_KEY), new VIntWritable(1));
		TreeMap<Double,Integer> yMap = new TreeMap<Double,Integer>();
		
		for (VIntWritable cat : weightMap.keySet()) {
			double yk = Math.exp(DiscriminantFunctionAlgorithm.predict(value,
					weightMap.get(cat)));
			yMap.put(yk, cat.get());
		}
		
		double predict = yMap.get(yMap.lastKey()) * classWidth + 2.5;
		System.out.println(predict + "(" + b + ")");
		
		diffSum += Math.abs(predict - b);
		trainSize++;
	}
	
	@Override
	public void cleanup(Context context) throws IOException,
			InterruptedException {
		context.write(new IntWritable(trainSize), new DoubleWritable(diffSum));
	}
}
