package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;
import java.util.Map.Entry;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import jp.co.yahoo.hadoop.ml.common.DiscriminantFunctionAlgorithm;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.Writable;

/**
 * A mapper for regression using Least Square Method.
 */
public class LeastSquaresMapper extends RegressionMapper {
	private double stepSize = 0.00000025;

	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		stepSize = conf.getFloat("sdm.step.size", (float) stepSize);
		super.setup(context);

	}

	@Override
	public void map(VIntWritable key, MapWritable value, Context context)
			throws IOException, InterruptedException {
		int b = key.get();
		value.put(new Text(CommonUtil.BIAS_KEY), new VIntWritable(1));
		double wx = DiscriminantFunctionAlgorithm.predict(value, weightMap);

		System.out.println(wx + "(" + b + ")");

		for (Entry<Writable, Writable> entry : value.entrySet()) {
			Text word = (Text) entry.getKey();
			int x = ((VIntWritable) entry.getValue()).get();
			DoubleWritable weightWritable = (DoubleWritable) weightMap
					.get(word);
			double diffWeight = -stepSize * (wx - b) * x;
			if (weightWritable == null) {
				weightMap.put(word, new DoubleWritable(diffWeight));
			} else {
				double w = weightWritable.get() + diffWeight;
				weightWritable.set(w);
			}
		}

		diffSum += Math.abs(wx - b);
		trainSize++;
	}
}
