package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;
import java.util.Map.Entry;

import jp.co.yahoo.hadoop.ml.common.CommonUtil;
import jp.co.yahoo.hadoop.ml.common.DiscriminantFunctionAlgorithm;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.Writable;

/**
 * A mapper for regression using Passive-Aggressive.
 */
public class PassiveAggressiveMapper extends RegressionMapper {
	private double genErr = 1; // e
	private double penalty = 0.01; // C

	@Override
	public void setup(Context context) throws IOException, InterruptedException {
		Configuration conf = context.getConfiguration();
		genErr = conf.getFloat("generalization.error.size", (float) genErr);
		penalty = conf.getFloat("penalty.size", (float) penalty);
		super.setup(context);
	}

	@Override
	public void map(VIntWritable key, MapWritable value, Context context)
			throws IOException, InterruptedException {
		int b = key.get();
		value.put(new Text(CommonUtil.BIAS_KEY), new VIntWritable(1));
		double wx = DiscriminantFunctionAlgorithm.predict(value, weightMap);

		System.out.println(wx + "(" + b + ")");

		int squareNormX = 0;
		for (Entry<Writable, Writable> entry : value.entrySet()) {
			int x = ((VIntWritable) entry.getValue()).get();
			squareNormX += x * x;
		}

		for (Entry<Writable, Writable> entry : value.entrySet()) {
			Text word = (Text) entry.getKey();
			int x = ((VIntWritable) entry.getValue()).get();
			DoubleWritable weightWritable = (DoubleWritable) weightMap
					.get(word);
			double diffWeight = Math.signum(b - wx)
					* Math.max(0, Math.abs(wx - b) - genErr)
					/ (squareNormX + 1 / 2 / penalty) * x;
			if (weightWritable == null) {
				weightMap.put(word, new DoubleWritable(diffWeight));
			} else {
				double w = weightWritable.get() + diffWeight;
				weightWritable.set(w);
			}
		}

		diffSum += Math.abs(wx - b);
		trainSize++;
	}
}