package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;
import java.util.Map.Entry;

import jp.co.yahoo.hadoop.common.IdentityReducer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.SequenceFileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.mapreduce.lib.output.TextOutputFormat;
import org.apache.hadoop.util.GenericOptionsParser;

/**
 * A MapReduce driver for ranking words of a weight vector
 */
public class RankWordsDriver {
	/**
	 * 
	 * The Mapper for ranking words of a weight vector
	 *
	 */
	public static class RankWordsMapper extends
			Mapper<NullWritable, MapWritable, DoubleWritable, Text> {
		@Override
		public void map(NullWritable key, MapWritable value, Context context)
				throws IOException, InterruptedException {
			for (Entry<Writable, Writable> entry : value.entrySet()) {
				Text word = (Text) entry.getKey();
				DoubleWritable score = (DoubleWritable) entry.getValue();
				context.write(score, word);
			}
		}
	}
	
	public static void main(String[] args) throws Exception {
		Configuration conf = new Configuration();

		String[] otherArgs = new GenericOptionsParser(conf, args)
				.getRemainingArgs();
		if (otherArgs.length < 2) {
			System.err
					.println("Usage: hadoop <input file> <output file>");
			System.exit(2);
		}

		Job job = new Job(conf, "rank words");
		job.setJarByClass(RankWordsDriver.class);
		job.setMapperClass(RankWordsMapper.class);
		job.setReducerClass(IdentityReducer.class);
		job.setInputFormatClass(SequenceFileInputFormat.class);
		job.setOutputFormatClass(TextOutputFormat.class);
		job.setOutputKeyClass(DoubleWritable.class);
		job.setOutputValueClass(Text.class);
		FileInputFormat.addInputPath(job, new Path(otherArgs[0]));
		FileOutputFormat.setOutputPath(job, new Path(otherArgs[1]));
		System.exit(job.waitForCompletion(true) ? 0 : 1);
	}
}
