package jp.co.yahoo.hadoop.ml.regression;

import java.io.IOException;
import java.util.Map.Entry;

import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.Reducer;

/**
 * A reducer for regression
 */
public class RegressionReducer extends
		Reducer<DoubleWritable, MapWritable, NullWritable, MapWritable> {
	private MapWritable mergedMap = new MapWritable();
	private double uSum = 0;
	private int mapSize = 0;
	private double maeSum = 0;
	
	@Override
	public void reduce(DoubleWritable key, Iterable<MapWritable> values,
			Context context) throws IOException, InterruptedException {
		double u = 1 / key.get();
		for (MapWritable value : values) {
			for (Entry<Writable, Writable> entry : value.entrySet()) {
				Text word = (Text) entry.getKey();
				DoubleWritable weight = (DoubleWritable) entry.getValue();
				DoubleWritable mergedWeight = (DoubleWritable) (mergedMap.get(word));
				double uw = weight.get() * u;
				if (mergedWeight == null) {
					mergedMap.put(word, new DoubleWritable(uw));
				} else {
					mergedWeight.set(mergedWeight.get() + uw);
				}
			}
			uSum += u;
			maeSum += key.get();
			mapSize++;
		}
	}
	
	@Override
	public void cleanup(Context context) throws IOException,
			InterruptedException {
		for (Entry<Writable, Writable> entry : mergedMap.entrySet()) {
			DoubleWritable weight = (DoubleWritable) entry.getValue();
			weight.set(weight.get() / uSum);
		}
		double mae = maeSum / mapSize;
		System.out.println("MAE: " + mae);
		context.write(NullWritable.get(), mergedMap);
	}
}